%%
% A hybrid image is the sum (or average) of a low-pass filtered version 
% of the one image and a high-pass filtered version of a second image. 
% The cutoff-frequency of each filter should be chosen empirically. 
% In the starter code, the cutoff frequency is controlled by changing 
% the standard deviation of the Gausian filter used in constructing the hybrid images.
% 
% For your favorite result, you should also illustrate the process through frequency analysis. Show
% the log magnitude of the Fourier transform of the two input images, the filtered images, and the
% hybrid image. In MATLAB, you can compute and display the 2D Fourier transform with:
%     imagesc(log(abs(fftshift(fft2(gray_image)))))
%     
% Try your algorithm on the sample image pairs provided. 
% And try creating a variety of types of hybrid images of your own 
% (change of expression, morph between different objects, 
% change over time, etc.). 
% The site has several examples that may inspire.
% 
% Construct and show the image pyramid of your hybrid images using 
% MATLAB function imresize(image, 0.75, bicubic) up to 5 levels.
% 
% Forbidden MATAB functions which you cannot use in your final code: 
% imfilter(), filter2(), conv2(), nlfilter(), colfilt().
%
% sungjoon.s.choi@gmail.com
% 
ccc;

%% 1. GENERATE HYBRID IMAGE
% 1. Load image to 'img'
ccc;
% select image
img_idx1 = 1;
img_idx2 = 2;
img_full_path1 = get_img_path(img_idx1);
img_full_path2 = get_img_path(img_idx2);

% load selected image
img1 = imread(img_full_path1);
img2 = imread(img_full_path2);
fprintf('[%s] + [%s] ARE LOADED. \n', img_full_path1, img_full_path2);

% 2. Do hybrid filtering using 'img1' and 'img2' 
%
% We will use [0~1] converted double image 
%
img1_double = im2double(img1);
img2_double = im2double(img2);

% Image for low pass and high pass 
img4lp_double = img2_double;
img4hp_double = img1_double;

lp_len = 21; lp_dev = 5;
lp_filter = (fspecial('gaussian', [lp_len, lp_len], lp_dev));
hp_len = 15; hp_dev = 5;  
hp_half = (hp_len-1)/2; 
hp_filter = padarray(1, [hp_half hp_half]) - (fspecial('gaussian', [hp_len, hp_len], hp_dev));

sum_lp = sum(sum(lp_filter));
sum_hp = sum(sum(hp_filter));
% fprintf('sum_lp: %.2e / sum_hp: %.2e \n', sum_lp, sum_hp);

plot_filter = 0;
if plot_filter
    fig = figure(2); clf;
    subaxes(fig, 1,2,1);
    imagesc(lp_filter); colorbar; axis square;
    title('Low pass filter');
    subaxes(fig, 1,2,2);
    imagesc(hp_filter); colorbar; axis square;
    title('High pass filter');
end

tic;
lp_img_double = my_imfilter(img4lp_double, lp_filter);
hp_img_double = my_imfilter(img4hp_double, hp_filter);
fprintf('Filtering finished. %.e sec \n', toc);
hyb_img_double = lp_img_double + hp_img_double;
resize_rate1 = 0.7;
r_hyb_img_double1 = imresize(hyb_img_double, resize_rate1, 'bicubic');
resize_rate2 = 0.4;
r_hyb_img_double2 = imresize(hyb_img_double, resize_rate2, 'bicubic');
resize_rate3 = 0.1;
r_hyb_img_double3 = imresize(hyb_img_double, resize_rate3, 'bicubic');

% plot 
fig = figure(3); clf; scrsz = get(0,'ScreenSize'); 
xstart = 0.2; ystart = 0.1;
xsize  = 0.5; ysize  = 0.5;
set(fig,'Position', [xstart*scrsz(3), ystart*scrsz(4), xsize*scrsz(3), ysize*scrsz(4)]);
subaxes(fig, 2,4,1);
imshow(img4lp_double);
title('Original image for Low-pass', 'FontSize', 15);
subaxes(fig, 2,4,2);
imshow(img4hp_double);
title('Original image for High-pass', 'FontSize', 15);
subaxes(fig, 2,4,3);
imshow(lp_img_double);
title('Low passed image', 'FontSize', 15);
subaxes(fig, 2,4,4);
imshow(hp_img_double+0.5);
title('High passed image + 0.5', 'FontSize', 15);
subaxes(fig, 2,4,5);
imshow(hyb_img_double);
title('Hybrid image', 'FontSize', 15);
subaxes(fig, 2,4,6);
imshow(r_hyb_img_double1);
title(sprintf('Resized Hybrid image (%.1f)', resize_rate1), 'FontSize', 15);
subaxes(fig, 2,4,7);
imshow(r_hyb_img_double2);
title(sprintf('Resized Hybrid image (%.1f)', resize_rate2), 'FontSize', 15);
subaxes(fig, 2,4,8);
imshow(r_hyb_img_double3);
title(sprintf('Resized Hybrid image (%.1f)', resize_rate3), 'FontSize', 15);

%% 2. Frequency analysis
% 
% For your favorite result, you should also illustrate the process through frequency analysis. Show
% the log magnitude of the Fourier transform of the two input images, the filtered images, and the
% hybrid image. In MATLAB, you can compute and display the 2D Fourier transform with:
%     imagesc(log(abs(fftshift(fft2(gray_image)))))
%  
g_img4lp_double = rgb2gray(img4lp_double);
g_img4hp_double = rgb2gray(img4hp_double);
g_hp_img_double = rgb2gray(lp_img_double);
g_lp_img_double = rgb2gray(hp_img_double);
g_hyb_img_double = rgb2gray(hyb_img_double);
g_r_hyb_img_double1 = rgb2gray(r_hyb_img_double1);
g_r_hyb_img_double2 = rgb2gray(r_hyb_img_double2);
g_r_hyb_img_double3 = rgb2gray(r_hyb_img_double3);

fig = figure(4); clf; scrsz = get(0,'ScreenSize'); 
xstart = 0.2; ystart = 0.1;
xsize  = 0.5; ysize  = 0.5;

set(fig,'Position', [xstart*scrsz(3), ystart*scrsz(4), xsize*scrsz(3), ysize*scrsz(4)]);
subaxes(fig, 2,3,1);
imagesc(log(abs(fftshift(fft2(g_img4lp_double)))))
title('Log magnitude of Original image4LP', 'FontSize', 15);

subaxes(fig, 2,3,2);
imagesc(log(abs(fftshift(fft2(g_img4hp_double)))))
title('Log magnitude of Original image4HP', 'FontSize', 15);

subaxes(fig, 2,3,3);
imagesc(log(abs(fftshift(fft2(g_hp_img_double)))))
title('Log magnitude of Low pass image', 'FontSize', 15);

subaxes(fig, 2,3,4);
imagesc(log(abs(fftshift(fft2(g_lp_img_double)))))
title('Log magnitude of High pass image', 'FontSize', 15);

subaxes(fig, 2,3,5);
imagesc(log(abs(fftshift(fft2(g_hyb_img_double)))))
title('Log magnitude of hybrid image', 'FontSize', 15);

subaxes(fig, 2,3,6);
imagesc(log(abs(fftshift(fft2(g_r_hyb_img_double1)))))
title('Log magnitude of scaled down hybrid image', 'FontSize', 15);

%%



