%%
% Implement an edge detection algorithm that uses
% - the zero-crossings of the LOG (Laplacian of Gaussian)
% - Simple finding of all zero crossings may yield a lot of noisy edges. So, you need to
% threshold the edge strength (slope and magnitude) for detecting only strong edges
% - Do not round the filter output to the nearest integer value before computing zero
% crossings.
%
% - And the zero-crossings of the LOBG (Laplacian of Bilateral Gaussian)
% - You can derive the LOBG equation and design the kernel similarly to LOG case
% ? Write a single algorithm that has two kernel options, LOG and LOGB.
% ? Test your algorithm on the test images by varying the parameters of each algorithm (Try 9x9
% kernel size with Gaussian std = 1.4).
% 
% ? Add Gaussian noise with std= 20 to the test images and repeat the test on them.
% ? Show and compare your results, and discuss on them. Give any ideas for improving the
% performance.
%
ccc;

%%
% 1. Load image to 'img'
img_idx = 2; % 1, 2, 3, 4, 5, 6
img_full_path = get_img_path(img_idx);
% load selected image
img_rgb    = imread(img_full_path);
img_double = im2double(img_rgb);
fprintf('[%s] LOADED. \n', img_full_path);

% 2. BiLoG Filter
gauss_len = 9;
gauss_dev = 1.4;
gauss_filter = (fspecial('gaussian', [gauss_len, gauss_len], gauss_dev));

blog_img_double = my_imbifilter(img_double, gauss_filter);

% 3. Find edge
switch(img_idx)
    case 1
        edge_param.patch_half_h = 1;
        edge_param.patch_half_w = 1;
        edge_param.slope_th  = .02;
        edge_param.zero_th   = .05;
    case 2
        edge_param.patch_half_h = 1;
        edge_param.patch_half_w = 1;
        edge_param.slope_th  = .02;
        edge_param.zero_th   = .05;
    case 3
        edge_param.patch_half_h = 1;
        edge_param.patch_half_w = 1;
        edge_param.slope_th  = .015;
        edge_param.zero_th   = .05;
    case 4
        edge_param.patch_half_h = 1;
        edge_param.patch_half_w = 1;
        edge_param.slope_th  = .01;
        edge_param.zero_th   = .08;
    case 5
        edge_param.patch_half_h = 1;
        edge_param.patch_half_w = 1;
        edge_param.slope_th  = .02;
        edge_param.zero_th   = .05;
    case 6
        edge_param.patch_half_h = 1;
        edge_param.patch_half_w = 1;
        edge_param.slope_th  = .01;
        edge_param.zero_th   = .05;
end

edge_img = find_edge(blog_img_double, edge_param);

% plot
if 1
    fig = figure(1); clf; scrsz = get(0,'ScreenSize'); 
    xstart = 0.1; ystart = 0.3;
    xsize  = 0.8; ysize  = 0.6;
    set(fig,'Position', [xstart*scrsz(3), ystart*scrsz(4), xsize*scrsz(3), ysize*scrsz(4)]);
    subaxes(fig, 1,3,1);
    imshow(img_double);
    subaxes(fig, 1,3,2);
    imshow(blog_img_double+0.5);
    title('BLoG filtered image with 0.5 bias');
    subaxes(fig, 1,3,3);
    imshow(edge_img);
    title('Black dots indicate edge');
end









