function edge_img = find_edge(log_img, param)
%
%  Fine edge from Laplacian of Gaussian filtered imgae 
%
[img_h, img_w, nr_channel] = size(log_img);


% params
if nargin == 1
    patch_half_h = 2;
    patch_half_w = 2;
    slope_th = .02;
    zero_th  = 0.05;
else
    patch_half_h = param.patch_half_h;
    patch_half_w = param.patch_half_w;
    slope_th     = param.slope_th;
    zero_th      = param.zero_th;
end


edge_img = ones(img_h, img_w);

for i = 1:nr_channel
    curr_img = log_img(:, :, i);
    
    % clf; imagesc(curr_img); axis off; axis square; colorbar; drawnow; pause;
    
    
    for h_curr = 1:img_h
        for w_curr = 1:img_w
            center_pixel = curr_img(h_curr, w_curr);
            
            
            h_fr = max(h_curr-patch_half_h, 1);
            h_to = min(h_curr+patch_half_h, img_h);
            w_fr = max(w_curr-patch_half_w, 1);
            w_to = min(w_curr+patch_half_w, img_w);
            
            pixel_h_prev = curr_img(h_fr:h_curr, w_curr);
            pixel_h_next = curr_img(h_curr:h_to, w_curr);
            pixel_w_prev = curr_img(h_curr, w_fr:w_curr);
            pixel_w_next = curr_img(h_curr, w_curr:w_to);
    
            % evaluate slopes  
            h_prev_max = max(pixel_h_prev);
            h_prev_min = min(pixel_h_prev);
            h_next_max = max(pixel_h_next);
            h_next_min = min(pixel_h_next);
            w_prev_max = max(pixel_w_prev);
            w_prev_min = min(pixel_w_prev);
            w_next_max = max(pixel_w_next);
            w_next_min = min(pixel_w_next);
            
            
            h_pos = ((0 - h_prev_min) > slope_th) && ((h_next_max - 0) > slope_th);
            h_neg = ((h_prev_max - 0) > slope_th) && ((0 - h_next_min) > slope_th);
            w_pos = ((0 - w_prev_min) > slope_th) && ((w_next_max - 0) > slope_th);
            w_neg = ((w_prev_max - 0) > slope_th) && ((0 - w_next_min) > slope_th);
            
            % check zero crossing + patch img
            if abs(center_pixel) < zero_th  ...
                    && (h_pos || h_neg || w_pos || w_neg)
                edge_img(h_curr, w_curr) = 0;
            end
        end
    end
end

