function fimg = my_bifilter2(gauss_filter, img)
%
% Bilateral filter for single channel image
%
% f    : filter
% img  : sigle channel matrix
%

dimg = double(img);

% size of filter
[hf, wf] = size(gauss_filter); 

% size of image
[hi, wi] = size(dimg);

% zeros pad to image
l = floor(max(hf, wf)/2);
lh = floor(hf/2);
lw = floor(wf/2);
zimg = wextend('2D', 'zpd', dimg, l);
temp_img = zimg;
for i = l+1 : hi+l
    for j = l+1 : wi+l
        patch_img = zimg(i-lh:i+lh, j-lw:j+lw);
        bi_patch = get_bipatch(patch_img, hf, wf);
        
        % for Debugging  
        % figure(99); imagesc(bipatch); axis off; axis square; caxis([0 1]); colorbar; pause; 
        
        % do Laplacian operator to 'patch_filter'
        lobg_filter = get_laplacian_filter(bi_patch.*gauss_filter);
        
        resp = patch_img.*lobg_filter;
        temp_img(i, j) = sum(sum(resp));
    end
end

fimg = temp_img(l+1:end-l, l+1:end-l);

% compute bilateral filter
function bipatch = get_bipatch(patch_img, hf, wf)
bipatch = ones(size(patch_img));
lh = floor(hf/2)+1;
lw = floor(wf/2)+1;
center_pixel = patch_img(lh, lw);
for i = 1:hf
    for j = 1:lw
        curr_pixel = patch_img(i, j);
        pixel_diff = abs(center_pixel-curr_pixel);
        bipatch(i, j) = exp(-pixel_diff*pixel_diff/10);
    end
end

% evaluate laplacian operator
function laplacian_filter = get_laplacian_filter(raw_filter)

laplacian_filter = raw_filter;
laplacian_operator = [0 1 0; 1 -4 1; 0 1 0];
% size of filter
[hf, wf] = size(raw_filter); 
zimg = wextend('2D', 'zpd', raw_filter, 1);
for i = 2:hf+1
    for j = 2:wf+1
        zpatch = zimg(i-1:i+1, j-1:j+1);
        fval = sum(sum(zpatch.*laplacian_operator));
        laplacian_filter(i-1, j-1) = fval;
    end
end

    




